/*
	Copyright 2003 Xavier Dunat

    This file is part of Suisse.

    Suisse is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Suisse is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Suisse; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
 * \file appariement.c
 * \author Xavier Dunat
 * \brief appariement des joueurs
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "joueur.h"
#include "appariement.h"
#include "ruleB.h"
#include "log.h"

double gain=1.0;
double nul =0.5;
double perte=0;

/**
 * calcul le factoriel de i
 *\param i
 *\return i!
 */
int fact(int i)
{
	int ret;
	if (i<0) exit(NOK);
	if (i==0) return (1);
	if (i==1) return (1);
	else ret=i*fact(i-1);
	return ret;
}

/**
 * Détermine qui est le joueur devant avoir le plus petit numéro d'appariement
 * \param struct player player0, struct player player1
 * \return 1 si le player0 est considéré comme meilleur que le player1, 2 si si c'est player2, 0 si indéterminé
*/
int bestPlayer(struct player player0, struct player player1)
{
	if (player0.numPairing==0) return(1); /*EXEMPT*/
	if (player1.numPairing==0) return(2); /*EXEMPT*/
	if (player0.score>player1.score) return(1);
	if (player1.score>player0.score) return(2);
	if (player0.elo>player1.elo) return(1);
	if (player1.elo>player0.elo) return(2);
	if (player0.title>player1.title) return(1);
	if (player1.title>player0.title) return(2);
	if (strcmp(player0.surname, player1.surname)<0) return(1);
	if (strcmp(player1.surname, player0.surname)<0) return(2);
	if (strcmp(player0.name, player1.name)<0) return(1);
	if (strcmp(player1.name, player0.name)<0) return(2);
	return (0);
}

int triListeTable(struct table *l_table, int nbTable)
{
	int i, j;
	struct table *saveTable;
	struct player *l_player, *Player;
	saveTable=malloc(nbTable*sizeof(struct table));
	l_player=malloc(nbTable*sizeof(struct player));
	Player=malloc(nbTable*sizeof(struct player));
	for (i=0; i<nbTable; i++)
	{
		saveTable[i]=l_table[i];
		if (bestPlayer(l_table[i].white, l_table[i].black)==1)
		{
			l_player[i]=l_table[i].white;
			Player[i]=l_player[i];
		}
		else
		{
			l_player[i]=l_table[i].black;
			Player[i]=l_player[i];
		}
	}
	triPlayer(l_player, nbTable);
	for (i=0; i<nbTable; i++)
	{
		j=0;
		while (l_player[j].numPairing!=Player[i].numPairing)
			j++;
		l_table[j]=saveTable[i];
	}
	free(saveTable);
	free(l_player);
	free(Player);
	return (OK);
}
/**
 * Tri la liste *list après 'numPosition' permutations
 * \param int numPosition
 * \param struct player *list
 * \param int nbPlayer
 */
int position(int numPosition, struct player *list, int nbPlayer)
{
	int q, p, i,j;
	struct player *oldListe;
	if (numPosition==1)
		return (OK);
	p=0;
	while (fact(p)<numPosition)
		p++;
	q=0;
	while ( !(q*(fact(p-1))< numPosition) || 
		!((q+1)*(fact(p-1))>= numPosition)) 
		q++;
	oldListe=malloc(nbPlayer*sizeof(struct player));
	for (i=0; i<nbPlayer-p; i++)
	{
		oldListe[i]=list[i];
	}
	oldListe[nbPlayer-p]=list[nbPlayer-p+q];
	j=0;
	for (i=0; i<p; i++)
	{
		if ( i!=q)
		{
			oldListe[nbPlayer-p+j+1]=list[nbPlayer-p+i];
			j++;
		}
	}
	position(numPosition-q*fact(p-1), oldListe, nbPlayer);
	for (i=0; i<nbPlayer;i++)
		list[i]=oldListe[i];
	free(oldListe);
	return (OK);
}

void permutePlayer(struct player *player0, struct player *player1)
{
	struct player a;
	a=*player0;
	*player0=*player1;
	*player1=a;
}

/**
* Détermine si un niveau est homogene
*
* \param struct player *list list de joueurs
*
* \param int number          nombre de joueurs
*
* \return int 0 si niveau homogene ou considéré comme tel, 1 sinon
*
*/
int homogeneLevel(struct player *list, int number)
{
	double scoreMax, scoreMin;
	int ret,i, nbNivSup;
	ret=0;
	scoreMax=0;
	scoreMin=g_round;
	for (i=0; i<number; i++)
	{
		if (scoreMin>list[i].score) scoreMin=list[i].score;
		if (scoreMax<list[i].score) scoreMax=list[i].score;
	}
	/*test niveau homogène*/
	if (scoreMin==scoreMax)
		return (0);
	/* nous avons un niveau hétérogène */
	/* calcul nombre de joueur de niveaux supérieurs */
	nbNivSup=0;
	for (i=0; i<number; i++)
		if (list[i].score>scoreMin)
			nbNivSup++;
	/* verifie si niveau hétérogène considéré homogène */
	if (nbNivSup>=(number/2.0))
		return(0);
	return (1);
}

int numbersExchange(int nb, int rang, int *numS1, int *numS2)
{
	int i, mini, maxi;
	*numS1=nb;
	*numS2=0;
	mini=maxi=0;
	i=1;
	if (rang==1)
		return (OK);
	for (i=2; i<=rang; i++)
	{
		if (*numS2==mini)
		{
			if (maxi<nb)
				maxi++;
			else
				mini++;
			*numS2=maxi;
			*numS1=nb-maxi+1;
		}
		else
		{
			(*numS2)--;
			(*numS1)--;
		}
	}
	return(OK);
}

/**
 * Apparie les joueurs
 *
 * \return 0 si OK
 *
 */
int appariement()
{
	int i, ret=OK;
	STD_LOG("appariement")("appariement ronde : %d", g_round);
//	g_listTable[g_round-1]=malloc(g_listPlayer.nbPlayer*sizeof(struct table));
	if (g_round>g_nbRoundValidated) {
		for (i=0; i<g_listPlayer.nbPlayer; i++)
		{
			g_listPlayer.Player[i].numAdv[g_round]=0;
		}
		if (g_round==1)
		{
			ret=appariementRound1();
		}
		else
		{
			ret=appariementOtherRound();
		}
		g_nbTable[g_round]=0;
		for (i=0; i<g_listPlayer.nbPlayer; i++)
		{
			STD_LOG("appariement")("Joueur %d (%s) contre Joueur %d", Pairing2Order(g_listPlayer.Player[i].numPairing)+1, g_listPlayer.Player[i].color[g_round]==WHITE ? "BLANC" : "NOIR", Pairing2Order(g_listPlayer.Player[i].numAdv[g_round])+1);
			if (g_listPlayer.Player[i].color[g_round]==WHITE)
			{
				g_listTable[g_round-1][g_nbTable[g_round]].white=g_listPlayer.Player[i];
				g_listTable[g_round-1][g_nbTable[g_round]].black=
					g_listPlayer.Player[Pairing2Order(g_listPlayer.Player[i].numAdv[g_round])];
				strcpy(g_listTable[g_round-1][g_nbTable[g_round]].result,"XXX-XXX");
				g_nbTable[g_round]++;
			} else
				if (g_listPlayer.Player[i].exempt[g_round]==1)
				{
					g_listTable[g_round-1][g_nbTable[g_round]].white=g_listPlayer.Player[i];
					g_listTable[g_round-1][g_nbTable[g_round]].black=playerExempt;
					strcpy(g_listTable[g_round-1][g_nbTable[g_round]].result,"1-F");
					g_nbTable[g_round]++;
				}
		}
		triListeTable(g_listTable[g_round-1], g_nbTable[g_round]);
		for (i=0; i < g_listPlayer.nbPlayer/2; i ++)
			STD_LOG("appariement")("n%d : %10s contre %10s", i+1, 
					g_listTable[g_round-1][i].white.surname,
					g_listTable[g_round-1][i].black.surname);
	}
//	free(g_listTable);
	return(ret);
}

/**
 * Calcul du nombre de joueur dans les sous groupes S1 et S2 à partir d'une liste de joueur d'un groupe
 * \param struct listOfPlayer list liste de joueurs triés
 * \param int *nbS1 nombre de joueurs dans S1
 * \param int *nbS2 nombre de joueurs dans S2
 */
int CalculNbS1S2(struct listOfPlayer list,
			     int *nbS1, int *nbS2)
{	
	int compt, homogene;
	double score, scoremin;
	compt=1;
	score=list.Player[0].score;
	while ( (compt < list.nbPlayer) && (list.Player[compt].score==score))
		compt++;
	if (compt==list.nbPlayer)
		homogene=0;
	else
	{
		scoremin=list.Player[list.nbPlayer-1].score;
		compt=0;
		while ( (compt < list.nbPlayer) && (list.Player[compt].score!=scoremin))
		{
			compt++;
		}
		if (compt>=(list.nbPlayer/2))
			homogene=0;
		else
			homogene=1;
	}
	if (0==homogene)
	{
		*nbS1=list.nbPlayer/2;
		*nbS2=list.nbPlayer-*nbS1;
	}
	else
	{
		*nbS1=compt;
		*nbS2=list.nbPlayer-compt;
	}
	return (OK);
}

int exchangePlayer(struct listOfPlayer S1, struct listOfPlayer S2,
		   int numExchange)
{
	int numS1=0, numS2=0, i;
	struct player pl;
	numbersExchange(S2.nbPlayer-1, numExchange, &numS1, &numS2);
	if (S2.nbPlayer!=S1.nbPlayer)
		numS1--;
	STD_LOG("exchangePlayer")("echange n%d : %d de S1 et %d de S2",
			numExchange, numS1, numS2);
	pl=S1.Player[numS1];
	S1.Player[numS1]=S2.Player[numS2];
	S2.Player[numS2]=pl;
	logList("S1", S1);
	logList("S2", S2);
	return (OK);
}
/**
* Test si on peut apparier les joueurs de S1 avec ceux de S2
* \param  *
* \return *
	  */
int testPairing(struct listOfPlayer S1, struct listOfPlayer S2,
		    struct perturbation Perturbation,
		    struct parameterOfRound param, int lastLevel,
		    int *tableNotPaired)
{
	int i, j,ret, nbApparie;
	STD_LOG("testPairing")("Entering");
	if ((0==param.p) && (0!=S1.nbPlayer))
		return(OK);
	nbApparie=0;
	j=0;
	for (i=0; i<S1.nbPlayer; i++)
	{
		if(testMatch(S1.Player[i],S2.Player[i],
			     Perturbation, param)!=OK)
		{
			if (S1.nbPlayer-nbApparie==param.p)
				return (i+1);
			else
			{
				tableNotPaired[j]=i;
				j++;
				nbApparie++;
			}
		}
		else
		{
			if (colorPref(S1.Player[i])*colorPref(S2.Player[i])>0)
			{
				if (Perturbation.low>0)
					Perturbation.low--;
				else
					Perturbation.hight--;
			}
		}
	}
	if (S2.nbPlayer==param.p+1)
	{
		STD_LOG("testPairing")("test B5");
		STD_LOG("testPairing")("ruleB5 : %d, nbPairing %d", param.ruleB5, param.p);
		STD_LOG("testPairing")("flotteur : %d", S2.Player[param.p].flotteur[g_round-1]);
		if (param.ruleB5 &&(S2.Player[param.p].flotteur[g_round-1]==-1))
		{
			STD_LOG("testPairing")("deuxieme descente");
			return (S2.nbPlayer);
		}
		if (param.ruleB6 &&(S2.Player[param.p].flotteur[g_round-2]==-1))
		{
			STD_LOG("testPairing")("regle B6 KO");
			return (S2.nbPlayer);
		}
	}
	if  (abs(S2.nbPlayer-S1.nbPlayer)==1) 
	{
		if (lastLevel)
		{
			int ret=alreadyExempt(S2.Player[S2.nbPlayer-1]);
			if (ret==0)
			{
				S2.Player[S2.nbPlayer-1].exempt[g_round]=1;
				S2.Player[S2.nbPlayer-1].resultat[g_round]=1.0;
				S2.Player[S2.nbPlayer-1].numAdv[g_round]=0;
				STD_LOG("testPairing")("%d exempt", S2.Player[S2.nbPlayer-1].numPairing);
			}
			else
			{
				STD_LOG("testPairing")("%d déjà exempt", S2.Player[S2.nbPlayer-1].numPairing);
				return(S2.nbPlayer);
			}
		}
	}
	return (OK);
}

/**
 * apparie les deux groupes de joueurs pour le tour en cours
 *
 * \param sous groupes S1 et S2
 *
 * \return 0 si OK
 */
int appariementS1S2(struct listOfPlayer S1, struct listOfPlayer S2,
				int *oldPermutation, int *oldExchange, 
				enum COLOR color, struct parameterOfRound param,
				int lastLevel, int *notPairing)
{
	int regleB5, regleB6, i,
		numS1, numS2, homogene, nbPermutePossible;
	int permutation, exchange, modifiedColor;
	int appariementOK, numPermute;
	int *tableNotPaired;
	struct player *S1S2, *save1, *save2, *newS2;
	struct perturbation Perturbation;
	if ((0==param.p) && (0!=S1.nbPlayer))
	{
		STD_LOG("appariementS1S2")("appariementS1S2 OK");
		for (i=0;i<S1.nbPlayer; i++)
		{
			STD_LOG("appariementS1S2")("table %d non appariée",i);
			notPairing[i]=i;
		}
		return(OK);
	}
	Perturbation.color=color;
	S1S2=malloc((S1.nbPlayer+S2.nbPlayer)*sizeof(struct player));
	save1=malloc(S1.nbPlayer*sizeof(struct player));
	save2=malloc(S2.nbPlayer*sizeof(struct player));
	newS2=malloc(S2.nbPlayer*sizeof(struct player));
	tableNotPaired=malloc(S1.nbPlayer*sizeof(int));
	regleB5=1;
	regleB6=1;
	nbPermutePossible=fact(S2.nbPlayer);
	for (i=0; i< S1.nbPlayer; i++)
	{
		S1S2[i]=S1.Player[i];
		save1[i]=S1.Player[i];
	}
	for (i=0; i< S2.nbPlayer; i++)
	{
		S1S2[S1.nbPlayer+i]=S2.Player[i];
		save2[i]=S2.Player[i];
		newS2[i]=S2.Player[i];
	}
	homogene=homogeneLevel(S1S2,S1.nbPlayer+S2.nbPlayer);
	hightColorPref(S1S2, S1.nbPlayer+S2.nbPlayer, param.x,
		           &Perturbation.low, &Perturbation.hight);
	if (param.x>0)
	{
		STD_LOG("appariementS1S2")("nb de perturbation x=%d",param.x);
		STD_LOG("appariementS1S2")("faible perturbée=%d",Perturbation.low);
		STD_LOG("appariementS1S2")("forte perturbée=%d",Perturbation.hight);
	}
	numS1=0; numS2=0;
	appariementOK=0;
	permutation=*oldPermutation;
	exchange=*oldExchange;
	modifiedColor=0;

	logList("S1", S1);
	logList("S2", S2);

	while ( (numPermute=testPairing(S1, S2, 
				Perturbation,param,
				lastLevel,
				tableNotPaired))!=OK)
	{
		STD_LOG("appariementS1S2")("numPermute=%d", numPermute);
		for (i=0; i<S2.nbPlayer; i++)
		{
			S2.Player[i]=newS2[i];
		}
		
		/* besoin de permutation dans S2 : C7 */
		permutation+=fact(S2.nbPlayer-numPermute);
		if (permutation<nbPermutePossible)
		{
			position(1+permutation, S2.Player, S2.nbPlayer);
			STD_LOG("appariementS1S2")("permutation : %d",permutation);
			logList("S1", S1);
			logList("S2", S2);
		}
		else
		{
			permutation=0;
			/* permutations insuffisants : C8 */
			/* si niveau homogene faire echanges */
			if (homogene!=0) {
				STD_LOG("appariementS1S2")("régle C8");
				STD_LOG("appariementS1S2")("niveau hétérogène pas d'échange");
			}
			if (exchange>(S2.nbPlayer-1)*(S2.nbPlayer-1))
			{
				STD_LOG("appariementS1S2")("régle C8");
				STD_LOG("appariementS1S2")("nb échange effectué maxi : %d",exchange);
			}
			if ((homogene==0) 
			     && (exchange<(S2.nbPlayer-1)*(S2.nbPlayer-1)))
			{
				for (i=0; i<S1.nbPlayer;i++)
				{
					S1.Player[i]=save1[i];
				}
				for (i=0; i<S2.nbPlayer;i++)
				{
					S2.Player[i]=save2[i];
					newS2[i]=save2[i];
				}
				exchange++;
				STD_LOG("appariementS1S2")("echange %d",exchange);
				exchangePlayer(S1, S2, exchange);
				for (i=0; i< S2.nbPlayer; i++)
					newS2[i]=S2.Player[i];
				continue;
			}
			else {
				/* echanges insuffisants : C9 */
				if (param.ruleB6==1)
				{
					for (i=0;i<S1.nbPlayer;i++)
						S1.Player[i]=save1[i];
					for (i=0;i<S2.nbPlayer;i++)
					{
						S2.Player[i]=save2[i];
						newS2[i]=save2[i];
					}
					exchange=0;
					permutation=0;
					param.ruleB6=0;
					STD_LOG("appariementS1S2")("C9 abandon B6");
					logList("S1", S1);
					logList("S2", S2);
					continue;
				}
				else
					if (param.ruleB5==1)
					{
						for (i=0;i<S1.nbPlayer;i++)
							S1.Player[i]=save1[i];
						for (i=0;i<S2.nbPlayer;i++)
						{
							S2.Player[i]=save2[i];
							newS2[i]=save2[i];
						}
						exchange=0;
						permutation=0;
						param.ruleB5=0;
						STD_LOG("appariementS1S2")("C9 abandon B5");
						logList("S1", S1);
						logList("S2", S2);
						continue;
					}
					else
					{
						/* C11 */
						if (param.x<S1.nbPlayer)
						{
							param.ruleC11=1;
							param.x++;
							hightColorPref(S1S2, S1.nbPlayer+S2.nbPlayer,
							  param.x, &Perturbation.low, &Perturbation.hight);
							permutation=0;
							exchange=0;
							param.ruleB5=1;
							param.ruleB6=1;
							STD_LOG("appariementS1S2")("C11 X=%d", param.x);
						}
						else
						{
							appariementOK=1;
							STD_LOG("appariementS1S2")("appariement S1-S2 impossible");
							break;
						}
					}
			}
		}
	}
	if (0==appariementOK)
	{
		for (i=0; i<S1.nbPlayer;i++)
		{
			int set=0, j, k=0;
			if (param.p!=S1.nbPlayer)
			{
				for (j=0; j<S1.nbPlayer-param.p; j++)
				{
					if (i==tableNotPaired[j])
						set=1;
				}
			}
			if (set==0)
			{
				setMatch(&g_listPlayer.Player[Pairing2Order(S1.Player[i].numPairing)],
					&g_listPlayer.Player[Pairing2Order(S2.Player[i].numPairing)]);
				STD_LOG("appariementS1S2")("%d contre %d",
						S1.Player[i].numPairing, S2.Player[i].numPairing);
			}
			else
			{
				notPairing[k]=i;
			}
		}
		STD_LOG("appariementS1S2")("lastLevel %d, nbS1 %d, nbS2 %d", 
				lastLevel, S1.nbPlayer, S2.nbPlayer);
		if (lastLevel && abs(S2.nbPlayer-S1.nbPlayer)==1)
		{
		/* prise en compte de l'exempt*/
			g_listPlayer.Player[Pairing2Order(S2.Player[S2.nbPlayer-1].numPairing)]
					  .resultat[g_round]=1.0;
			g_listPlayer.Player[Pairing2Order(S2.Player[S2.nbPlayer-1].numPairing)]
				          .flotteur[g_round]=-1;
			g_listPlayer.Player[Pairing2Order(S2.Player[S2.nbPlayer-1].numPairing)]
					.flotteurTheorique[g_round]=-1;
			g_listPlayer.Player[Pairing2Order(S2.Player[S2.nbPlayer-1].numPairing)]
					  .exempt[g_round]=1;
			g_listPlayer.Player[Pairing2Order(S2.Player[S2.nbPlayer-1].numPairing)]
			        	  .color[g_round]=NOPLAY;
			g_listPlayer.Player[Pairing2Order(S2.Player[S2.nbPlayer-1].numPairing)]
				          .colorTheorique[g_round]=NOPLAY;
			g_listPlayer.Player[Pairing2Order(S2.Player[S2.nbPlayer-1].numPairing)]
				          .numAdv[g_round]=0;
			g_listPlayer.Player[Pairing2Order(S2.Player[S2.nbPlayer-1].numPairing)]
				          .numAdvTheorique[g_round]=0;
			STD_LOG("appariementS1S2")("player %d exempt", S2.Player[S2.nbPlayer-1].numPairing);
		}
	}
	free(S1S2);
	free(save1);
	free(save2);
	free(newS2);
	free(tableNotPaired);
	if (appariementOK==0)
	{
		*oldPermutation=permutation;
		*oldExchange=exchange;
	}
	STD_LOG("appariementS1S2")("return %s",appariementOK ? "KO" : "OK");
	return(appariementOK);
}

/**
 * apparies les joueurs pour la premiere ronde
 *
 * \return 0 si OK
 */
static int appariementRound1()
{
	int nbS1, nbS2, i;
	struct player *S1, *S2;
	struct listOfPlayer listLevel;
	listLevel.nbPlayer=0;
	listLevel.Player=malloc(sizeof(struct player));
	for (i=0; i< g_listPlayer.nbPlayer; i++)
		if (g_listPlayer.Player[i].paired[1]==1)
		{
			listLevel.Player[listLevel.nbPlayer]=g_listPlayer.Player[i];
			listLevel.nbPlayer++;
			listLevel.Player=realloc(listLevel.Player, 
				(listLevel.nbPlayer+1)*sizeof(struct player));
		}

	STD_LOG("appariementRound1")("Nb joueur %d", g_listPlayer.nbPlayer);
	CalculNbS1S2(listLevel, &nbS1, &nbS2);
	S1=malloc(nbS1*sizeof(struct player));
	S2=malloc(nbS2*sizeof(struct player));
	for (i=0; i<nbS1; i++)
		S1[i]=listLevel.Player[i];
	for (i=0; i<nbS2; i++)
		S2[i]=listLevel.Player[i+nbS1];
	for (i=1; i<=g_listPlayer.nbPlayer; i++)
	{
		g_listPlayer.Player[i].color[1]=WHITE;
		g_listPlayer.Player[i].numAdv[1]=0;
	}
	DET_LOG("appariementRound1")("nb S1=%d, nb S2=%d", nbS1, nbS2);
	for (i=0; i < nbS1; i++)
	{
		g_listPlayer.Player[Pairing2Order(S1[i].numPairing)].numAdv[1]=S2[i].numPairing;
		g_listPlayer.Player[Pairing2Order(S1[i].numPairing)].numAdvTheorique[1]=S2[i].numPairing;
		
		g_listPlayer.Player[Pairing2Order(S2[i].numPairing)].numAdv[1]=S1[i].numPairing;
		g_listPlayer.Player[Pairing2Order(S2[i].numPairing)].numAdvTheorique[1]=S1[i].numPairing;
		if ( ((i%2==0) && (g_colorNum1==WHITE)) || ((i%2==1) && (g_colorNum1==BLACK)))
		{
			g_listPlayer.Player[Pairing2Order(S1[i].numPairing)].color[1]=WHITE;
			g_listPlayer.Player[Pairing2Order(S2[i].numPairing)].color[1]=BLACK;
		}
		else
		{
			g_listPlayer.Player[Pairing2Order(S1[i].numPairing)].color[1]=BLACK;
			g_listPlayer.Player[Pairing2Order(S2[i].numPairing)].color[1]=WHITE;
		}
	}
	for (i=0; i<g_listPlayer.nbPlayer; i++)
	{
		if(g_listPlayer.Player[i].paired[1]==0)
		{
			g_listPlayer.Player[i].color[1]=NOPLAY;
			g_listPlayer.Player[i].numAdv[1]=0;
		}
		else
			if ((g_listPlayer.Player[i].numAdv[1]==0) &&
				(g_listPlayer.Player[i].color[1]!=NOPLAY))
			{
				g_listPlayer.Player[i].color[1]=NOPLAY;
				g_listPlayer.Player[i].flotteur[1]=-1;
				g_listPlayer.Player[i].flotteurTheorique[1]=-1;
				g_listPlayer.Player[i].score=1.0;
				g_listPlayer.Player[i].exempt[1]=1;
				g_listPlayer.Player[i].resultat[1]=1.0;
			}
		g_listPlayer.Player[i].colorTheorique[1]=g_listPlayer.Player[i].color[1];
	}
	free(S1);
	free(S2);
	free(listLevel.Player);
	STD_LOG("appariementRound1")("return OK");
	return (OK);
}

/**
 * Détermine les sous groupes S1 et S2 à partir d'une liste d'un groupe
 *
 * \param struct listOfPlayer list
 *
 * \param struct listOfPlayer *S1
 *
 * \param struct listOfPlayer *S2
 *
 * \return OK
 */
int calculS1S2(struct listOfPlayer list, 
			   struct listOfPlayer *S1,
			   struct listOfPlayer *S2)
{
	int i;
	/* C5 */
	CalculNbS1S2(list, &S1->nbPlayer, &S2->nbPlayer);
	S1->Player=malloc((S1->nbPlayer)*sizeof(struct player));
	S2->Player=malloc((S2->nbPlayer)*sizeof(struct player));
	STD_LOG("calculS1S2")("%d joueurs :",list.nbPlayer);
	for (i=0; i< S1->nbPlayer; i++)
	{
		S1->Player[i]=list.Player[i];
	}
	for (i=0; i< S2->nbPlayer; i++)
	{
		S2->Player[i]=list.Player[i+S1->nbPlayer];
	}
	STD_LOG("calculS1S2")("return OK");
	return (OK);
}

/**
 * apparies les joueurs d'un niveau
 *
 * \param struct listOfPlayer g_listPlayerLevel liste des joueurs a apparier
 * \param struct listOfPlayer listNotPaired liste des joueurs non appariés
 *\param int lastLevel 1 si c'est le dernier niveau
 * \return 0 si OK
 */
static int appariementLevel(struct listOfPlayer g_listPlayerLevel,
			    struct listOfPlayer *listNotPaired,
			    int lastLevel)
{
	struct listOfPlayer S1, S2, oldS1, oldS2, S1Init, S2Init;
	struct parameterOfRound param;
	int i, Permutation, Exchange,
		oldPermutation, oldExchange,
		PermutationGeneral, ExchangeGeneral, nbPermutePossible,
		nbPlayerPaired=0, residualOK;
	int *notPairingC14;
	enum COLOR color;

	param.ruleB5=1;
	param.ruleB6=1;
	param.ruleC11=0;
	STD_LOG("appariementLevel")("Entering");
	if (g_listPlayerLevel.nbPlayer>0)
	{
		/* appariement du niveau entier */
		/* C5 */
		calculS1S2(g_listPlayerLevel, &S1, &S2);
		nbPermutePossible=fact(S2.nbPlayer);
		if (copyListOfPlayer(S1, &oldS1)) {
			STD_LOG("appariementLevel")("Erreur copie S1");
			return(NOK);
		}
		if (copyListOfPlayer(S2, &oldS2)) {
			STD_LOG("appariementLevel")("Erreur copie S2");
			return(NOK);
		}
		param.p=S1.nbPlayer;
		notPairingC14=malloc(S1.nbPlayer*sizeof(int));
		STD_LOG("appariementLevel")("NbS1=%d, nbS2=%d", S1.nbPlayer, S2.nbPlayer);
		/* C6 à C9*/
		PermutationGeneral=0;
		ExchangeGeneral=0;
		Permutation=0;
		Exchange=0;
		residualOK=0;
		param.x=calculX(S1, S2, &color);
		do {
			int app;
			if (residualOK!=0) {
				STD_LOG("appariementLevel")("défait niveau");
				copyListOfPlayer(oldS1, &S1);
				copyListOfPlayer(oldS2, &S2);
				PermutationGeneral++;
				position(PermutationGeneral, S2.Player, S2.nbPlayer);
				if (PermutationGeneral>nbPermutePossible)
				{
					/*CAUTION*/
				}
			}
			residualOK=1;
			/* appariement niveau non résiduel */
			oldPermutation=PermutationGeneral;
			oldExchange=ExchangeGeneral;
			STD_LOG("appariementLevel")("permutation=%d, echange=%d", PermutationGeneral, ExchangeGeneral);
			app=appariementS1S2(S1, S2, 
					 &PermutationGeneral, &ExchangeGeneral,
					 color, param,
					 lastLevel, notPairingC14);
	/* Regle C10, cas où un nouvel appariement n'a pas pu etre fait */
			if ((app!=OK) && ((oldPermutation!=0) || (oldExchange!=0)))
			{
				STD_LOG("appariementLevel")("C10");
				PermutationGeneral=0;
				ExchangeGeneral=0;
				appariementS1S2(S1, S2,
			                  &PermutationGeneral, &ExchangeGeneral,
				 	  color, param, lastLevel, notPairingC14);
				/* abandonner critère B6 puis B5 */
				if (param.ruleB6==1)
				{
					STD_LOG("appariementLevel")("B6");
					param.ruleB6=0;
					continue;
				}
				else
					if (param.ruleB5==1)
					{
						STD_LOG("appariementLevel")("B5");
						param.ruleB5=0;
						continue;
					}
			}
			if (app!=OK)
			{
/*				if (param.x<S1.nbPlayer)
				{
				STD_LOG("appariementLevel")("C11");
					param.x++;
					continue;
				}
				else
				{*/
				if (param.p>0)
				{
					param.p--;
					STD_LOG("appariementLevel")("C14\np=%d",param.p);
					continue;
				}
				break;
			}
			else
			{
				residualOK=0;
				nbPlayerPaired+=2*param.p;
				listNotPaired->nbPlayer=0;
				/* tous les joueurs sont appariés*/
				if (S2.nbPlayer==param.p)
				{
					listNotPaired->nbPlayer=0;
				}
/* les joueurs restant de S2 sont mis dans la liste des non appariés */
                                if (S2.nbPlayer>S1.nbPlayer)
				{
					listNotPaired->nbPlayer=S2.nbPlayer-S1.nbPlayer;
					for (i=0; i< listNotPaired->nbPlayer; i++)
						listNotPaired->Player[i]=S2.Player[i+S1.nbPlayer];
				}
				
	/* vérifie si tous les joueurs de S1 sont appariés */
				for (i=0;i<S1.nbPlayer-param.p;i++)
				{
				   listNotPaired->Player[S2.nbPlayer-S1.nbPlayer+i]= S1.Player[i];
				   listNotPaired->Player[S2.nbPlayer-S1.nbPlayer+i+1 ]= S2.Player[i];
				}
			       listNotPaired->nbPlayer+=(S1.nbPlayer-param.p)*2;
				
				if (lastLevel && abs(S2.nbPlayer-S1.nbPlayer)==1)
					nbPlayerPaired+=1;
			}
			/* appariement du niveau residuel */
			if ((listNotPaired->nbPlayer>1) && (param.p==S1.nbPlayer))
			{
				Permutation=0;
				Exchange=0;
				STD_LOG("appariementLevel")("appariement niveau résiduel");
				calculS1S2(*listNotPaired, &S1, &S2);
				param.p=S1.nbPlayer;
				int x=param.x;
				param.x=calculX(S1, S2, &color);
				residualOK=appariementS1S2(S1, S2,
			                           &Permutation, &Exchange,
						   color, param, lastLevel,
						   notPairingC14);
				if (residualOK!=OK)
				{
					STD_LOG("appariementLevel")("Niveau résiduel impossible à apparier.");
				}
				param.x=x;
				nbPlayerPaired+=2*S1.nbPlayer;
				if (S1.nbPlayer==S2.nbPlayer)
					listNotPaired->nbPlayer=0;
				if (S2.nbPlayer>S1.nbPlayer)
				{
					listNotPaired->nbPlayer=S2.nbPlayer-S1.nbPlayer;
					for (i=0; i< listNotPaired->nbPlayer; i++)
						listNotPaired->Player[i] = S2.Player[i+S1.nbPlayer];
				}
				if (S2.nbPlayer<S1.nbPlayer)
				{
					listNotPaired->nbPlayer=S1.nbPlayer-S2.nbPlayer;
					for (i=0; i< listNotPaired->nbPlayer;i++)
						listNotPaired->Player[i]=S1.Player[i+S2.nbPlayer];
				}
			}
		}
		while (residualOK!=0);
		STD_LOG("appariementLevel")("apparié : %d non app %d", nbPlayerPaired, listNotPaired->nbPlayer);
		if (S1.nbPlayer>0) free(S1.Player);
		free(S2.Player);
		free(oldS1.Player);
		free(oldS2.Player);
		free(notPairingC14);
	}
	STD_LOG("appariementLevel")("return OK");
	return (OK);
}

/**
 * apparies les joueurs pour le tour en cours
 * \return 0 si OK
 */
static int appariementOtherRound()
{
	double level;
	int i, playNextLevel, nbPlayerPaired, lastLevel, numLevel,
		appariementOK;
	struct listOfPlayer list, listNotPaired, *listLevel;
	level=(double)(g_round-1);
	numLevel=0;
	listLevel=malloc((g_nbRound*2+1)*sizeof(struct listOfPlayer));
	listNotPaired.nbPlayer=0;
	list.nbPlayer=0;
	nbPlayerPaired=0;
	playNextLevel=0;
	lastLevel=0;
	while ((level>=0) && (lastLevel==0)) {
		STD_LOG("appariementOtherRound")("level %3.1f",level);
		/* ajouts des joueurs du niveaux precedents non appariés */
		list.Player=malloc(sizeof(struct player));
		for (list.nbPlayer=0; list.nbPlayer<listNotPaired.nbPlayer; list.nbPlayer++)
		{
			list.Player[list.nbPlayer] = listNotPaired.Player[list.nbPlayer];
			list.Player = realloc(list.Player, (list.nbPlayer+2)*sizeof(struct player));
		}
		/* recuperation des joueurs avec (level) points */
		for (i=0; i<g_listPlayer.nbPlayer; i++)
		{
			if ((g_listPlayer.Player[i].score==level) &&
			    (g_listPlayer.Player[i].paired[g_round]==1) && (g_listPlayer.Player[i].out[g_round]==0))
			{
				list.Player[list.nbPlayer]=g_listPlayer.Player[i];
				list.nbPlayer++;
				list.Player=realloc(list.Player, (list.nbPlayer+1)*sizeof(struct player));
			}
		}
		listNotPaired.Player=malloc(list.nbPlayer*sizeof(struct player));
		triPlayer(list.Player, list.nbPlayer);
		if (g_listPlayer.nbPlayer-nbPlayerPaired-g_nbNotPaired-g_nbPlayerOut[g_round]==list.nbPlayer)
			lastLevel=1;
		else
		{
			STD_LOG("appariementOtherRound")("nb joueur: %d\nnb joueur Appariés : %d",
				   g_listPlayer.nbPlayer,nbPlayerPaired);
			STD_LOG("appariementOtherRound")("nb joueur sorti : %d", g_nbPlayerOut[g_round]);
			STD_LOG("appariementOtherRound")("nb joueur non appariés : %d\nnb joueur essai d'appariement : %d", g_nbNotPaired, list.nbPlayer);
		}
		STD_LOG("appariementOtherRound")("lastLevel=%d",lastLevel);
		appariementOK=appariementLevel(list, &listNotPaired, lastLevel);
		if (OK==appariementOK)
			nbPlayerPaired+=list.nbPlayer-listNotPaired.nbPlayer;
		listLevel[numLevel].Player=malloc(sizeof(struct player));
		listLevel[numLevel].nbPlayer=0;
		for (i=0; i<list.nbPlayer; i++)
		{
			int j,paired=1;
			for (j=0; j<listNotPaired.nbPlayer; j++)
			{
				if (listNotPaired.Player[j].numPairing==
						list.Player[i].numPairing)
					paired=0;
			}
			if (paired)
			{
				listLevel[numLevel].Player=realloc(listLevel[numLevel].Player,
					(listLevel[numLevel].nbPlayer+1)*sizeof(struct player));
				listLevel[numLevel].Player[listLevel[numLevel].nbPlayer]=list.Player[i];
				listLevel[numLevel].nbPlayer++;
			}
		}
		if (lastLevel) {
			if (appariementOK==OK && listNotPaired.nbPlayer==1)
			{
				int numParing=Pairing2Order(listNotPaired.Player[0].numPairing);
				g_listPlayer.Player[numParing].exempt[g_round]=1;
				g_listPlayer.Player[numParing].resultat[g_round]=1.0;
				g_listPlayer.Player[numParing].numAdv[g_round]=0;
				STD_LOG("appariementOtherRound")("Joueur %d exempt ronde %d", listNotPaired.Player[0].numPairing, g_round);
				g_listPlayer.Player[numParing].score++;
			}
			while (appariementOK!=OK)
			{
				STD_LOG("appariementOtherRound")("Régle C13");
				numLevel--;
				listLevel[numLevel].Player=realloc(listLevel[numLevel].Player,
								(listLevel[numLevel].nbPlayer+
								listLevel[numLevel+1].nbPlayer)*
								sizeof(struct player));
				for (i=0; i< listLevel[numLevel+1].nbPlayer;i++)
					listLevel[numLevel].Player[listLevel[numLevel].nbPlayer+i]=listLevel[numLevel+1].Player[i];
				appariementOK=appariementLevel(listLevel[numLevel], &listNotPaired, lastLevel);
				free (listLevel[numLevel+1].Player);
			}
		}
		free(list.Player);
		level-=0.5;
	}
	for (i=0; i< g_listPlayer.nbPlayer; i++)
		g_listPlayer.Player[i].colorTheorique[g_round]=g_listPlayer.Player[i].color[g_round];
	for (i=0; i< numLevel; i++)
		free(listLevel[i].Player);
	free(listLevel);
	STD_LOG("appariementOtherRound")("return OK");
	return(OK);
}
