/*
	Copyright 2003 Xavier Dunat

    This file is part of Suisse.

    Suisse is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Suisse is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Suisse; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * \file departage.c
 * \author Xavier Dunat
 * \brief different systeme de departage
 */

#include <stdio.h>
#include <stdlib.h>
#include "joueur.h"
#include"departage.h"
#include "log.h"

int calculCumulatif(struct player *listPlayer)
{
	int numPlayer, i, j, roundSet;
	double cumul;
	
	roundSet=nbRoundSet();
	STD_LOG("calculCumulatif")("Entering ronde %d, nbPlayer=%d", roundSet, g_listPlayer.nbPlayer);
	for (numPlayer=0; numPlayer<g_listPlayer.nbPlayer; numPlayer++)
	{
		DET_LOG("calculCumulatif")("joueur %s", listPlayer[numPlayer].surname);
		listPlayer[numPlayer].cumulatif=0.0;
		for (i=1; i<=roundSet; i++) {
			cumul=0.0;
			for (j=1; j<=i; j++) {
				if (listPlayer[numPlayer].resultat[j]!=NO_SCORE)
					cumul+=listPlayer[numPlayer].resultat[j];
			}

			listPlayer[numPlayer].cumulatif+=cumul;
			if (1==listPlayer[numPlayer].exempt[i] && listPlayer[numPlayer].resultat[j]!=NO_SCORE)
				listPlayer[numPlayer].cumulatif-=1;
		}
	}
	STD_LOG("calculCumulatif")("return OK");
	return (OK);
}
int calculScoreAjuste(struct player *listPlayer)
{
	int i, numPlayer, roundSet;
	roundSet=nbRoundSet();
	
	STD_LOG("calculScoreAjuste")("entering roundSet=%d", roundSet);
	for (numPlayer=0; numPlayer<g_listPlayer.nbPlayer; numPlayer++)
	{
		listPlayer[numPlayer].scoreAjuste=0.0;
		for (i=1; i<=roundSet; i++)
			if (listPlayer[numPlayer].resultat[i]!=NO_SCORE)
				if (listPlayer[numPlayer].numAdv[i]==0)
					listPlayer[numPlayer].scoreAjuste+=0.5;
				else
					listPlayer[numPlayer].scoreAjuste+=listPlayer[numPlayer].resultat[i];
	}
	return(OK);
}
int calculSolkoff(struct player *listPlayer)
{
	int i, numPlayer, roundSet;
	roundSet=nbRoundSet();
	STD_LOG("calculSolkoff")("entering roundSet=%d", roundSet);
	calculScoreAjuste(listPlayer);
	for (numPlayer=0; numPlayer<g_listPlayer.nbPlayer; numPlayer++)
	{
		listPlayer[numPlayer].solkoff=0.0;
		for (i=1; i<=roundSet; i++)
			if (listPlayer[numPlayer].numAdv[i]==0 && listPlayer[numPlayer].resultat[i]!=NO_SCORE)
			{
				if (listPlayer[numPlayer].exempt[i]==1)
					listPlayer[numPlayer].solkoff+=0.5*g_nbRound;
				else
					if (0!=listPlayer[numPlayer].numAdvTheorique[i])
						listPlayer[numPlayer].solkoff+=0.5*g_listPlayer.Player[Pairing2Order(g_listPlayer.Player[numPlayer].numAdvTheorique[i])].scoreAjuste;
			}
			else
				listPlayer[numPlayer].solkoff+=g_listPlayer.Player[Pairing2Order(g_listPlayer.Player[numPlayer].numAdv[i])].scoreAjuste;
	}
	return(OK);
}

/**
 * classement la liste list en fonction niveau 0 : nb point, niveau 1 cumulatif, niveau 2 Solkoff
 *\param struct player *list la liste des joueurs
 *\param int level niveau choisi
 *\param int min rang dans la liste du premier joueur a trier
 *\param int max rang dans la liste du dernier joueur a trier
 *\return OK si classement r�ussi, -1 si level incorrecte
 */
static int classementLevel(struct player *list, int level, int min, int max)
{
	int compt1, compt2, index;
	switch (level)
	{
	case 0 :
		for (compt1=min; compt1 < max; compt1++)
			for (compt2=compt1+1; compt2 <=max; compt2++)
				if (list[compt1].score < list[compt2].score)
					inversePlayer(list, compt1, compt2);
		for (compt1=min; compt1 < max; compt1++)
		{
			index=0;
			while ((list[compt1].score== list[compt1+index+1].score) && (compt1+index+1<=max))
				  index++;
			if (index>0)
				classementLevel(list, 1, compt1, compt1+index);
		}
		break;
	case 1 :
		for (compt1=min; compt1 < max; compt1++)
			for (compt2=compt1+1; compt2 <=max; compt2++)
				if (list[compt1].cumulatif < list[compt2].cumulatif)
					inversePlayer(list, compt1, compt2);
		for (compt1=min; compt1 < max; compt1++)
		{
			index=0;
			while ((list[compt1].cumulatif==list[compt1+index+1].cumulatif) &&
			   (compt1+index+1<=max))
				  index++;
			if (index>0)
				classementLevel(list, 2, compt1, compt1+index);
		}
		break;
	case 2 :
		for (compt1=min; compt1 < max; compt1++)
			for (compt2=compt1+1; compt2 <=max; compt2++)
				if (list[compt1].solkoff < list[compt2].solkoff)
					inversePlayer(list, compt1, compt2);
		break;
	default :
		return (-1);
	}
	return (OK);
}
	
int classement(struct player *listCopie)
{
	int i;
	STD_LOG("classement")("entering");
	calculCumulatif(listCopie);
	calculSolkoff(listCopie);
	classementLevel(listCopie, 0, 0, g_listPlayer.nbPlayer);
	STD_LOG("classement")("return OK");
	return (OK);
}

int grilleAmericaine(char *nameFile)
{
	int i,j, numAdv, ret, roundSet;
	FILE *file;
	
	roundSet=nbRoundSet();
	STD_LOG("grilleAmericaine")("calcul pour %d rondes", roundSet);
	struct player *playerList=malloc(g_listPlayer.nbPlayer*sizeof(struct player));
/*	for (i=0; i<g_listPlayer.nbPlayer; i++) {
		playerList[i]=g_listPlayer.Player[i];
	}*/
	copyStructPlayer(g_listPlayer.Player, playerList, g_listPlayer.nbPlayer);
	classement(playerList);

	if ((file=fopen(nameFile,"w"))!=NULL)
	{
	for (i=0; i<g_listPlayer.nbPlayer; i++)
	{
		fprintf(file, "%2d %10s ", i+1, playerList[i].surname);
		for (j=1; j<=roundSet; j++)
		{
			if (playerList[i].numAdv[j]==0)
			{
				if (playerList[i].numAdvTheorique[j]==0 && playerList[i].out[j]==0)
				{
					fprintf(file, " EXE ");
				}
				else
				{
					numAdv=playerList[i].numAdvTheorique[j];
					if (numAdv==0)
					{
						fprintf(file, "     ");
					}
					else
					{
						if (playerList[i].resultat[j]==1) {
							fprintf(file,">");
						}
						else {
							fprintf(file,"<");
						}
						fprintf(file, "%2d", getOrder(numAdv, playerList)+1 );
						if (playerList[i].colorTheorique[j]==WHITE) {
							fprintf(file, "B ");
						}
						else{
							fprintf(file, "N ");
						}
					}
				}
			}
			else
			{
				if (playerList[i].resultat[j]==0.0) {
					fprintf(file,"-");
				}
				if (playerList[i].resultat[j]==0.5) {
					fprintf(file,"=");
				}
				if (playerList[i].resultat[j]==1.0) {
					fprintf(file,"+");
				}
				numAdv=playerList[i].numAdv[j];
				fprintf(file,"%2d",getOrder(numAdv, playerList)+1);
				if (playerList[i].colorTheorique[j]==WHITE) {
					fprintf(file, "B ");
				}
				else {
					fprintf(file, "N ");
				}
			}
		}
		fprintf(file,"\n");
	}
	fclose(file);
	ret=OK;
	}
	else {
		STD_LOG("grilleAmericaine")("ouverture impossible");
		ret=1;
	}
	free(playerList);
	return(ret);
}
