/*
	Copyright 2003 Xavier Dunat

    This file is part of Suisse.

    Suisse is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Suisse is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Suisse; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "joueur.h"
#include "log.h"
#include "pass.h"
#include "iosuisse.h"
#ifdef DATABASE
#include "libpq-fe.h"
#endif

int readPlayer(char *fileName)
{
	FILE *fileRead;
	char name[21], surname[21];
	int title, elo, i;
	g_listPlayer.nbPlayer=0;
	fileRead=fopen(fileName, "r");
	if (!fileRead)
		return (1);
	while (!feof(fileRead))
	{
		fscanf(fileRead, "%s %s %d %d\n", surname, name, &elo, &title);
		strcpy (g_listPlayer.Player[g_listPlayer.nbPlayer].name, name);
		strcpy (g_listPlayer.Player[g_listPlayer.nbPlayer].surname, surname);
		g_listPlayer.Player[g_listPlayer.nbPlayer].elo=elo;
		g_listPlayer.Player[g_listPlayer.nbPlayer].score=0.0;
		g_listPlayer.Player[g_listPlayer.nbPlayer].title=title;
		for (i=1; i<=g_nbRound; i++) {
		     g_listPlayer.Player[g_listPlayer.nbPlayer].paired[i]=1;
		     g_listPlayer.Player[g_listPlayer.nbPlayer].flotteur[i]=0;
		     g_listPlayer.Player[g_listPlayer.nbPlayer].exempt[i]=0;
		     g_listPlayer.Player[g_listPlayer.nbPlayer].numAdv[i]=0;
		     g_listPlayer.Player[g_listPlayer.nbPlayer].numAdvTheorique[i]=0;
		     g_listPlayer.Player[g_listPlayer.nbPlayer].resultat[i]=NO_SCORE;
		}
		STD_LOG("readPlayer")("nom(%d) : %s",g_listPlayer.nbPlayer, g_listPlayer.Player[g_listPlayer.nbPlayer].surname);
		g_listPlayer.nbPlayer++;
		g_listPlayer.Player=realloc(g_listPlayer.Player, 
				(g_listPlayer.nbPlayer+1)*sizeof(struct player));
		listPlayerDisplay=realloc(listPlayerDisplay, 
				(g_listPlayer.nbPlayer+1)*sizeof(struct player));
	}
	STD_LOG("readPlayer")("%d joueurs", g_listPlayer.nbPlayer);
/*	g_nbTable=g_listPlayer.nbPlayer/2;
	if (g_nbTable*2!=g_listPlayer.nbPlayer) g_nbTable++; /* au cas ou nbPlayer est impair */
	fclose(fileRead);
	return (0);
}

int writeFileTournament(char *fileName)
{
	FILE *fileWrite;
	char name[21], surname[21];
	int title, elo, i, j;
	fileWrite=fopen(fileName, "w");
	if (!fileWrite)
	{
		printf("probleme d'ouverture du fichier\n");
		return(NOK);
	}
	fprintf(fileWrite, "%d\n", g_nbRound);
	fprintf(fileWrite, "%d\n", g_listPlayer.nbPlayer);
	fprintf(fileWrite, "%d\n", g_nbRoundValidated);
	for (i=0; i<g_listPlayer.nbPlayer; i++) {
		fprintf(fileWrite, "%s\n", g_listPlayer.Player[i].surname);
		fprintf(fileWrite, "%s\n", g_listPlayer.Player[i].name);
		fprintf(fileWrite, "%d\n", g_listPlayer.Player[i].title);
		fprintf(fileWrite, "%d %d\n", g_listPlayer.Player[i].elo, g_listPlayer.Player[i].numPairing);
		fprintf(fileWrite, "%04.1f\n", g_listPlayer.Player[i].score);
		for (j=1; j<=g_nbRound; j++) {
			fprintf(fileWrite, "%04.1f\n", g_listPlayer.Player[i].resultat[j]);
			fprintf(fileWrite, "%d %d %d\n", g_listPlayer.Player[i].paired[j], g_listPlayer.Player[i].exempt[j],
			    g_listPlayer.Player[i].out[j]);
			fprintf(fileWrite, "%d %d %d\n", g_listPlayer.Player[i].numAdv[j], g_listPlayer.Player[i].numAdvTheorique[j], g_listPlayer.Player[i].flotteur[j]);
			fprintf(fileWrite, "%d %d\n", g_listPlayer.Player[i].color[j], g_listPlayer.Player[i].colorTheorique[j]);
		}
	}
	for (i=0; i<g_nbRound; i++) {
		fprintf(fileWrite, "%d\n", g_nbTable[i]);
		for (j=0; j<g_nbTable[i]; j++) {
			fprintf(fileWrite, "%d %d\n", g_listTable[i][j].white.numPairing, g_listTable[i][j].black.numPairing);
			fprintf(fileWrite, "%s\n", g_listTable[i][j].result);
		}
	}
	if (fclose(fileWrite)) {
		printf("pb de fermeture\n");
		return (NOK);
	}
	return(OK);
}

int readFileTournament(char *fileName)
{
	FILE *fileWrite;
	int i, j, black, white;
	char result[5], score[5];
	
	fileWrite=fopen(fileName, "r");
	if (!fileWrite)
	{
		printf("probleme d'ouverture du fichier\n");
		return(NOK);
	}
	fscanf(fileWrite, "%d\n", &g_nbRound);
	fscanf(fileWrite, "%d\n", &g_listPlayer.nbPlayer);
	fscanf(fileWrite, "%d\n", &g_nbRoundValidated);
	DET_LOG("readFileTournament")("nb ronde=%d, nb joueur=%d, nb table=%d", g_nbRound, g_listPlayer.nbPlayer, g_nbTable);

	g_listPlayer.Player=realloc(g_listPlayer.Player, (g_listPlayer.nbPlayer+1)*sizeof(struct player));
	listPlayerDisplay=realloc(listPlayerDisplay, (g_listPlayer.nbPlayer+1)*sizeof(struct player));
	g_nbPlayerOut=malloc((g_nbRound+1)*sizeof(int));
	for (i=0; i<=g_nbRound;i++)
		g_nbPlayerOut[i]=0;

	for (i=0; i<g_listPlayer.nbPlayer; i++) {
		fscanf(fileWrite, "%s\n", g_listPlayer.Player[i].surname);
		fscanf(fileWrite, "%s\n", g_listPlayer.Player[i].name);
		fscanf(fileWrite, "%d\n", &g_listPlayer.Player[i].title);
		fscanf(fileWrite, "%d %d\n", &g_listPlayer.Player[i].elo, &g_listPlayer.Player[i].numPairing);
		fscanf(fileWrite, "%s\n", score);
		g_listPlayer.Player[i].score=atof(score);
		for (j=1; j<=g_nbRound; j++)
		{
			fscanf(fileWrite, "%s\n", result);
			g_listPlayer.Player[i].resultat[j]=atof(result);
			fscanf(fileWrite, "%d %d %d\n", &g_listPlayer.Player[i].paired[j], &g_listPlayer.Player[i].exempt[j], 
			      &g_listPlayer.Player[i].out[j]);
			if (g_listPlayer.Player[i].out[j]!=0) {
				g_nbPlayerOut[j]++;
				STD_LOG("readFileTournament")("ronde %d, joueur %d, nb_sorti %d", j, i, g_nbPlayerOut[j]);
			}
			fscanf(fileWrite, "%d %d %d\n", &g_listPlayer.Player[i].numAdv[j], &g_listPlayer.Player[i].numAdvTheorique[j], &g_listPlayer.Player[i].flotteur[j]);
			fscanf(fileWrite, "%d %d\n", &g_listPlayer.Player[i].color[j], &g_listPlayer.Player[i].colorTheorique[j]);
		}
	}
	g_listTable=malloc((g_nbRound+1)*sizeof(struct table*));
	g_nbTable=malloc((g_nbRound+1)*sizeof(int));
	for (i=0; i<g_nbRound; i++) {
		fscanf(fileWrite, "%d\n", g_nbTable[i]);
		DET_LOG("readFileTournament")("ronde %d, nbTable=%d",i+1, g_nbTable[i]);
		g_listTable[i]=malloc((g_nbTable[i]+1)*sizeof(struct table));
		for (j=0; j<g_nbTable[i]; j++) {
			fscanf(fileWrite, "%d %d\n", &white, &black);
			g_listTable[i][j].white=g_listPlayer.Player[Pairing2Order(white)];
			g_listTable[i][j].black=g_listPlayer.Player[Pairing2Order(black)];
			DET_LOG("readFileTournament")("table %d blanc=%s(%d), noir=%s(%d)", j+1, 
				g_listTable[i][j].white.surname, g_listTable[i][j].white.numPairing, g_listTable[i][j].black.surname, g_listTable[i][j].black.numPairing);
			fscanf(fileWrite, "%s\n", g_listTable[i][j].result);
		}
	}
	for (i=0; i<g_listPlayer.nbPlayer; i++) {
		STD_LOG("readFileTournament")("%s num=%d score=%f", g_listPlayer.Player[i].surname, g_listPlayer.Player[i].numPairing, g_listPlayer.Player[i].score);
	}
	insertPlayerInList();
	if (fclose(fileWrite)) {
		printf("pb de fermeture\n");
		return (NOK);
	}
	return(OK);
}
#ifdef DATABASE
int readDataBase()
{
	char section_code[3];
	char query_string[256];
	PGconn *conn;
	PGresult *res;
	int i;
	conn=PQconnectdb("dbname=/home/developpeur/suisse/FFE.DBF");
	if (PQstatus(conn) == CONNECTION_BAD)
	{
		fprintf(stderr, "connection to database failed.\n");
		fprintf(stderr, "%s", PQerrorMessage(conn));
		return (-1);
	}
	printf("Enter a section code:");
	scanf("%s", section_code);
	sprintf(query_string, "SELECT name FROM sectionname WHERE code=%s", section_code);
	res=PQexec(conn, query_string);
	if (PQresultStatus(res) != PGRES_TUPLES_OK)
	{
		fprintf(stderr, "SELECT query failed.\n");
		PQclear(res);
		PQfinish(conn);
		return (-1);
	}
	for (i=0; i<PQntuples(res); i++)
		printf("%s\n", PQgetvalue(res, i, 0));
	PQclear(res);
	PQfinish(conn);
	return (0);
}
#endif
