/*
	Copyright 2003 Xavier Dunat

    This file is part of Suisse.

    Suisse is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Suisse is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Suisse; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 *\file joueur.c
 *\author Xavier Dunat
 *\brief définition des joueurs
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "pass.h"
#include "joueur.h"
#include "iosuisse.h"
#include "indem.h"
#include "appariement.h"
#include "ruleB.h"
#include "departage.h"
#include "log.h"

struct player playerExempt;

void initExempt()
{
	int i;
	strcpy(playerExempt.name, "EXEMPT");
	strcpy(playerExempt.surname, "EXEMPT");
	playerExempt.score=0.0;
	playerExempt.numPairing=0;
/*	for (i=0;i<=g_nbRound; i++) {
		playerExempt.
	}*/
}
/**
 * inverse deux valeurs entières
 * \param int a premiere valeur à inverser
 * \param int b deuxième valeur à inverser
 */
void permute(int a, int b)
{
	int c;
	c=a;
	a=b;
	b=c;
}

/**
 * Donne le numéro d'ordre interne correspondant au numéro d'appariement
 * \param int numPairing numéro d'appariement
 * \return numOrder numéro d'ordre
 */
int Pairing2Order(int numPairing)
{
	int numOrder=0;
	while (g_listPlayer.Player[numOrder].numPairing!=numPairing)
		numOrder++;
	return (numOrder);
}

int getOrder(int numPairing, struct player *list)
{
	int numOrder=0;
	while (list[numOrder].numPairing!=numPairing)
		numOrder++;
	return (numOrder);
}

/**
 * permute deux joueurs dans une liste de joueur
 *
 * \param struct player *list liste de joueur
 *
 * \param int player1 no d'ordre du premier joueur à permuter
 *
 * \param int player2 no d'ordre du second joueur à permuter
 *
 * \return OK
 */
int inversePlayer(struct player *list,int player1, int player2)
{
	struct player playerA;
	playerA=list[player1];
	list[player1]=list[player2];
	list[player2]=playerA;
	return (OK);
}

/**
 * défini la rencontre entre deux joueurs et leur attribue les paramètres correspondant
 *\param struct player *player0
 *\param struct player *player1
 *\return OK
 */
int setMatch(struct player *player0, struct player *player1)
{
	player0->numAdv[g_round]=player1->numPairing;
	player0->numAdvTheorique[g_round]=player1->numPairing;
	player1->numAdv[g_round]=player0->numPairing;
	player1->numAdvTheorique[g_round]=player0->numPairing;
	if (player0->score>player1->score)
	{
		player0->flotteur[g_round]=-1;
		player0->flotteurTheorique[g_round]=-1;
		player1->flotteur[g_round]=1;
		player1->flotteurTheorique[g_round]=1;
	} else
	if (player0->score<player1->score)
	{
		player0->flotteur[g_round]=1;
		player0->flotteurTheorique[g_round]=1;
		player1->flotteur[g_round]=-1;
		player1->flotteurTheorique[g_round]=-1;
	} else {
		player0->flotteur[g_round]=0;
		player0->flotteurTheorique[g_round]=0;
		player1->flotteur[g_round]=0;
		player1->flotteurTheorique[g_round]=0;
	}
	getColorMatch(player0, player1);
	return (OK);
}

int setPlayerWithParam(char *surname, char *name, int elo, int title)
{
	int i;
	STD_LOG("setPlayerWithParam")("entering");
        strcpy(g_listPlayer.Player[g_listPlayer.nbPlayer].surname, surname);
        strcpy(g_listPlayer.Player[g_listPlayer.nbPlayer].name, name);
	g_listPlayer.Player[g_listPlayer.nbPlayer].elo=elo;
	g_listPlayer.Player[g_listPlayer.nbPlayer].title=title;
	g_listPlayer.Player[g_listPlayer.nbPlayer].score=0.0;
	for (i=1; i<=g_nbRound; i++)
	{
		g_listPlayer.Player[g_listPlayer.nbPlayer].resultat[i]=NO_SCORE;
		g_listPlayer.Player[g_listPlayer.nbPlayer].paired[i]=1;
		g_listPlayer.Player[g_listPlayer.nbPlayer].flotteur[i]=0;
		g_listPlayer.Player[g_listPlayer.nbPlayer].flotteurTheorique[i]=0;
		g_listPlayer.Player[g_listPlayer.nbPlayer].exempt[i]=0;
	}
	g_listPlayer.nbPlayer++;
	g_listPlayer.Player=realloc(g_listPlayer.Player, (g_listPlayer.nbPlayer+1)*sizeof(struct player));
	listPlayerDisplay =realloc(listPlayerDisplay, (g_listPlayer.nbPlayer+1)*sizeof(struct player));
	return(OK);
}

void reinitResult(struct player *Player, int round) {
	Player->flotteur[round]=Player->flotteurTheorique[round];
	Player->color[round]=Player->colorTheorique[round];
	Player->numAdv[round]=Player->numAdvTheorique[round];
	Player->exempt[round]=0;
}
int getResult(int numApp, char *Result)
{
	int numAdv;
	numApp=Pairing2Order(numApp);
	numAdv=Pairing2Order(g_listPlayer.Player[numApp].numAdvTheorique[g_round]);
	
	double result;
	if (g_listPlayer.Player[numApp].resultat[g_round]!=NO_SCORE){
		DET_LOG("getResult")("Joueur %d, ancien score %3.1f", numApp, g_listPlayer.Player[numApp].resultat[g_round]);
		g_listPlayer.Player[numApp].score-=g_listPlayer.Player[numApp].resultat[g_round];
		reinitResult(&g_listPlayer.Player[numApp], g_round);
	}
	if (g_listPlayer.Player[numAdv].resultat[g_round]!=NO_SCORE) {
		DET_LOG("getResult")("Joueur %d, ancien score %3.1f", numAdv, g_listPlayer.Player[numAdv].resultat[g_round]);
		g_listPlayer.Player[numAdv].score-=g_listPlayer.Player[numAdv].resultat[g_round];
		reinitResult(&g_listPlayer.Player[numAdv], g_round);
	}
	if (strcmp(Result,"1-F")==0)
	{
		g_listPlayer.Player[numApp].resultat[g_round]=1.0;
		g_listPlayer.Player[numApp].exempt[g_round]=1;
		g_listPlayer.Player[numApp].flotteur[g_round]=-1;
		g_listPlayer.Player[numApp].numAdv[g_round]=0;
		g_listPlayer.Player[numApp].color[g_round]=NOPLAY;
		g_listPlayer.Player[numApp].score+=1.0;

		g_listPlayer.Player[numAdv].resultat[g_round]=0.0;
		g_listPlayer.Player[numAdv].flotteur[g_round]=0;
		g_listPlayer.Player[numAdv].numAdv[g_round]=0;
		g_listPlayer.Player[numAdv].color[g_round]=NOPLAY;
		/*g_listPlayer.Player[numAdv].score+=0.0;*/
		return (OK);
	}
	if (strcmp(Result,"F-1")==0)
	{
		g_listPlayer.Player[numApp].resultat[g_round]=0.0;
		g_listPlayer.Player[numApp].flotteur[g_round]=0;
		g_listPlayer.Player[numApp].numAdv[g_round]=0;
		g_listPlayer.Player[numApp].color[g_round]=NOPLAY;
		/*g_listPlayer.Player[numApp].score+=0.0;*/
		
		g_listPlayer.Player[numAdv].resultat[g_round]=1.0;
		g_listPlayer.Player[numAdv].flotteur[g_round]=-1;
		g_listPlayer.Player[numAdv].exempt[g_round]=1;
		g_listPlayer.Player[numAdv].numAdv[g_round]=0;
		g_listPlayer.Player[numAdv].color[g_round]=NOPLAY;
		g_listPlayer.Player[numAdv].score+=1.0;
		return(OK);
	}
	if (strcmp(Result,"F-F")==0)
	{
		g_listPlayer.Player[numApp].resultat[g_round]=0.0;
		g_listPlayer.Player[numApp].flotteur[g_round]=0;
		g_listPlayer.Player[numApp].numAdv[g_round]=0;
		g_listPlayer.Player[numApp].color[g_round]=NOPLAY;
		/*g_listPlayer.Player[numApp].score+=0.0;*/
		
		g_listPlayer.Player[numAdv].resultat[g_round]=0.0;
		g_listPlayer.Player[numAdv].flotteur[g_round]=0;
		g_listPlayer.Player[numAdv].numAdv[g_round]=0;
		g_listPlayer.Player[numAdv].color[g_round]=NOPLAY;
		/*g_listPlayer.Player[numAdv].score+=0.0;*/
		return (OK);
	}
	if(strcmp(Result,"0-0")==0)
	{
		g_listPlayer.Player[numApp].resultat[g_round]=0.0;
		g_listPlayer.Player[numAdv].resultat[g_round]=0.0;
		return(OK);
	}
	if(strcmp(Result,"1-0")==0)
	{
		result=1.0;
	}
	if(strcmp(Result,"0-1")==0)
	{
		result=0.0;
	}
	if(strcmp(Result,"0.5-0.5")==0)
	{
		result=0.5;
	}
	g_listPlayer.Player[numApp].resultat[g_round]=result;
	g_listPlayer.Player[numAdv].resultat[g_round]=1.0-result;
	g_listPlayer.Player[numApp].score+=g_listPlayer.Player[numApp].resultat[g_round];
	g_listPlayer.Player[numAdv].score+=g_listPlayer.Player[numAdv].resultat[g_round];
	return(OK);
}

/**
 * attribue un numéro d'appariement dans l'ordre croissant de la force théorique des joueurs
 *\param struct listOfPlayer list
 *\return 0 si OK
 */
int numPairing(struct listOfPlayer list)
{
	int i,j;
	triPlayer(list.Player, list.nbPlayer);
	for (i=0; i<list.nbPlayer; i++)
	{
		list.Player[i].numPairing=i+1;
		for (j=1; j<=g_nbRound; j++)
			list.Player[i].out[j]=0;
	}
	return (OK);
}

int getOutOnePlayer(int numApp, int numRoundMin, int numRoundMax)
{
	int i, nb;
	nb=Pairing2Order(numApp);
	for (i=numRoundMin; i<=numRoundMax; i++)
	{
		g_listPlayer.Player[nb].paired[i]=0;
		g_listPlayer.Player[nb].numAdv[i]=0;
		g_listPlayer.Player[nb].numAdvTheorique[i]=0;
		g_listPlayer.Player[nb].exempt[i]=0;
		g_listPlayer.Player[nb].resultat[i]=0.0;
		g_listPlayer.Player[nb].out[i]=1;
		g_listPlayer.Player[nb].color[i]=NOPLAY;
		g_nbPlayerOut[i]++;
	}
//	g_nbPlayerOut++;
}

int copyStructPlayer (struct player *source, struct player *dest, int nbPlayer)
{
	int i, j;
	for (i=0; i<nbPlayer; i++) {
		strcpy(dest[i].surname, source[i].surname);
		strcpy(dest[i].name, source[i].name);
		dest[i].title	= source[i].title;
		dest[i].elo  	= source[i].elo;
		dest[i].score	= source[i].score;
		dest[i].numPairing = source[i].numPairing;
		dest[i].cumulatif  = source[i].cumulatif;
		dest[i].solkoff    = source[i].solkoff;
		dest[i].scoreAjuste = source[i].scoreAjuste;
		for (j=0; j<g_nbRound; j++) {
			dest[i].resultat[j]	= source[i].resultat[j];
			dest[i].numAdv[j]	= source[i].numAdv[j];
			dest[i].numAdvTheorique[j]	= source[i].numAdvTheorique[j];
			dest[i].color[j]		= source[i].color[j];
 			dest[i].colorTheorique[j]	= source[i].colorTheorique[j];
			dest[i].flotteur[j]		= source[i].flotteur[j];
			dest[i].flotteurTheorique[j]	= source[i].flotteurTheorique[j];
			dest[i].paired[j]		= source[i].paired[j];
			dest[i].exempt[j]		= source[i].exempt[j];
			dest[i].out[j]			= source[i].out[j];
		}
	}
	return (OK);
}

int copyListOfPlayer (struct listOfPlayer source, struct listOfPlayer *dest)
{
	int i;
	dest->nbPlayer=source.nbPlayer;
	dest->Player=malloc((dest->nbPlayer)*sizeof(struct player));
/*	if (dest->Player==NULL) {
		STD_LOG("copyListOfPlayer")("Memory full");
		return(NOK);
	}
	for (i=0; i< source.nbPlayer; i++)
		dest->Player[i]=source.Player[i];*/
	copyStructPlayer(source.Player, dest->Player, source.nbPlayer);
	return (OK);
}

/**
 * calcul le nombre de ronde ou tous les r�sultats sont saisis
 *\return nombre de ronde saisie
 */
int nbRoundSet()
{
	int round=0, roundTest=1, i;
	int allSet=1;
	while (roundTest<=g_nbRound && allSet==1)	{
		for (i=0; i<g_listPlayer.nbPlayer; i++) {
			if (g_listPlayer.Player[i].resultat[roundTest]==NO_SCORE)
				allSet=0;
		}
		if (allSet==1) round=roundTest;
		roundTest++;
	}
	return(round);
}
