/*
	Copyright 2003 Xavier Dunat

    This file is part of Suisse.

    Suisse is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Suisse is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Suisse; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <stdlib.h>
#include "joueur.h"
#include "appariement.h"
#include "ruleB.h"
#include "log.h"

/**
 *\file ruleB.c
 *\author Xavier Dunat
 *\brief Régles B
 */

/**
 * Détermine si deux joueur se sont déjà recontrés
 * \param struct player player0
 * \param struct player player1
 * \return 1 si déjà rencontré, 0 sinon
 */
int alreadyPlay(struct player player0, struct player player1)
{
	int i, ret;
	ret=0;
	i=0;
	while (i<g_round && ret==0)
	{
		if (player0.numAdv[i]==player1.numPairing)
			ret=1;
		i++;
	}
	return ret;
}

/**
 * Détermine si un joueur a déjà été exempt
 * \param struct player m_player
 * \return 1 si déjà exempt, 0 sinon
 */
int alreadyExempt(struct player m_player)
{
	int i;
	i=1;
	DET_LOG("alreadyExempt")("test exempt joueur %d",m_player.numPairing);
	while (i<g_round)
	{
		if ( ((m_player.numAdv[i]==0) && (m_player.resultat[i]==1.0)) ||
		     (m_player.exempt[i]==1))
		{
			DET_LOG("alreadyExempt")("joueur %d ne peut plus être exempt.",
				m_player.numPairing);
			return 1;
		}
		i++;
	}
	return 0;
}

/**
 * calcul le parametre X (nombre de table perturbée)
 * \param struct listOfPlayer S1
 * \param struct listOfPlayer S2
 * \return valeur de X
 */
int calculX(struct listOfPlayer S1, struct listOfPlayer S2, enum COLOR *color)
{
	int i, j, b, n, noPref, q, x;
	b=0;
	n=0;
	for (i=0; i <S1.nbPlayer; i++)
	{
		if (colorPref(S1.Player[i])>0) n++;
		if (colorPref(S1.Player[i])<0) b++;
	}
	for (i=0; i <S2.nbPlayer; i++)
	{
		if (colorPref(S2.Player[i])>0) n++;
		if (colorPref(S2.Player[i])<0) b++;
	}
	if ((S1.nbPlayer+S2.nbPlayer)%2==0)
		q=(S1.nbPlayer+S2.nbPlayer)/2;
	else
		q=(S1.nbPlayer+S2.nbPlayer)/2+1;
	if (n>b)
	{
		*color=BLACK;
		x=n-q;
	}
	else
	{
		*color=WHITE;
		x=b-q;
	}
	return (x);
}

/**
 * \determine la couleur préférencielle d'un joueur
 * \param struct player Player
 * \return -1 pour blanche, 1 noire
 */
int colorPref(struct player Player)
{
	int i, color, lastColor, nbWhite, nbBlack;
	color=0;
	lastColor=NOPLAY;
	nbBlack=0;
	nbWhite=0;
	for (i=1; i<g_round; i++)
	{
		if (Player.color[i]==BLACK)
			nbBlack++;
		if (Player.color[i]==WHITE)
			nbWhite++;
	}
	/* Ne peut avoir 3 fois la même couleur */
	for (i=g_round-1; i>=1; i--) {
		if (Player.color[i]!=NOPLAY) {
			if (lastColor==NOPLAY) {
				lastColor=Player.color[i];
				DET_LOG("colorPref")("Joueur %d, ronde %d, couleur %d", Player.numPairing, i, lastColor);
			}
			else {
				if (Player.color[i]==lastColor) {
					STD_LOG("colorPref")("Joueur %d, ronde %d, pref absolue = %d", Player.numPairing, i, -lastColor*3);
					return (-lastColor*3);
				} else break;
			}
		}
	}
	if (nbWhite-nbBlack>1)
		return (3);
	if (nbWhite-nbBlack<-1)
		return (-3);
	if (g_round>2 && Player.color[g_round-1]==Player.color[g_round-2])
		return (-3*Player.color[g_round-1]);
	if (nbWhite-nbBlack==1)
		return (2);
	if (nbWhite-nbBlack==-1)
		return (-2);
	i=g_round-1;
	while ((Player.color[i]==NOPLAY) && i>0)
		i--;
	return (-(int)Player.color[i]);
}

/** 
 * Calcul du nombre de table pertubée
 * \param struct player *list tableau des joueurs du niveau
 * \param int nbPlayer nombre de joueur
 * \param int X nombre de table perturbée au total
 * \param int nbLowModif nombre de préférence faible à perturbée
 * \param int nbLowModif nombre de préférence forte à perturbée
 */
int hightColorPref(struct player *list, int nbPlayer, int X, 
		   int *nbLowModif, int *nbHightModif)
{
	int nbHightBlack, nbHightWhite, nbLowBlack, nbLowWhite, nbNoPref,i;
	int nbBlack, nbWhite;
	int *prefColor;
	nbNoPref=nbHightBlack=nbHightWhite=nbLowBlack=nbLowWhite=0;
	nbBlack=nbWhite=0;
	(*nbHightModif)=(*nbLowModif)=0;
	prefColor=malloc(nbPlayer*sizeof(int));
	if (X==0) return(OK);
	DET_LOG("hightColorPref")("nombre de table perturbée %d", X);
	
	for (i=0; i<nbPlayer;i++)
	{
		prefColor[i]=colorPref(list[i]);
		switch (prefColor[i])
		{
		case -2 : 
			{
				nbHightBlack++;	nbBlack++;
				break;
			}
		case -1 : 
			{
				nbLowBlack++; nbBlack++;
				break;
			}
		case 0 :
			{
				nbNoPref++;
				break;
			}
		case 1 :
			{
				nbLowWhite++; nbWhite++;
				break;
			}
		case 2 :
			{
				nbHightWhite++; nbWhite++;
				break;
			}
		}
	}
	free(prefColor);
	DET_LOG("hightColorPref")("nombre de préf forte noire    : %d", nbHightBlack);
	DET_LOG("hightColorPref")("nombre de préf faible noire   : %d", nbLowBlack);
	DET_LOG("hightColorPref")("nombre de préf forte blanche  : %d", nbHightWhite);
	DET_LOG("hightColorPref")("nombre de préf faible blanche : %d", nbLowWhite);
	if (nbBlack>nbWhite)
	{
		if (nbLowBlack>=X)
		{
			*nbLowModif=X;
			return(OK);
		}
		else
		{
			*nbLowModif=nbLowBlack;
			*nbHightModif=X-nbLowBlack;
		}
	}
	else
	{
		if (nbLowWhite>=X)
		{
			*nbLowModif=X;
			return(OK);
		}
		else
		{
			*nbLowModif=nbLowWhite;
			*nbHightModif=X-nbLowWhite;
		}
	}
	return(OK);
}

/**
 * Vérifie si deux joueurs peuvent être apparier ensemble
 *
 *\param struct player player1 premier joueur
 *\param struct player player2 second joueur
 *\param perturbation Perturbation permutation autorisé
 *\param parameterOfRound paramêtre standard de la ronde
 *\return OK si c'est possible NOK sinon
 */
/*                      testMatch                   */
/* v�rifie si player1 et player2 peuvent �tre       */
/* apparier ensemble                                */
/* entree : player1 et player2                      */
/*          color - couleur pouvant �tre chang�     */
/*          *low - nbre de personne ayant une       */
/*          pr�f�rence faible pouvant �tre permut�  */
/*          *hight - idem avec une pr�f�rence forte */
/* retour : OK si possibilit� de les apparier       */
/*          autre sinon                             */
/****************************************************/
int testMatch(struct player player1, struct player player2,
	      struct perturbation Perturbation,
	      struct parameterOfRound param)
{
	int colorPref1, colorPref2;
	/* verification B1 */
	STD_LOG("testMatch")("essai %d contre %d : ",
				player1.numPairing,
				player2.numPairing);
	DET_LOG("testMatch")("perturbation faible=%d, forte =%d", Perturbation.low, Perturbation.hight);
	if (alreadyPlay(player1, player2))
	{
		STD_LOG("testMatch")("DJ",player1.surname,player2.surname);
		return (1);
	}
	/* verification B4 */
	colorPref1=colorPref(player1);
	colorPref2=colorPref(player2);
	DET_LOG("testMatch")("couleur préf joueur1=%d", colorPref1);
	DET_LOG("testMatch")("couleur préf joueur2=%d", colorPref2);
	if (player1.score==player2.score)
	{
		player1.flotteur[g_round]=0;
		player2.flotteur[g_round]=0;
	}
	else
		if (player1.score>player2.score)
		{
			player1.flotteur[g_round]=1;
			player2.flotteur[g_round]=-1;
		}
		else
		{
			player1.flotteur[g_round]=-1;
			player2.flotteur[g_round]=1;
		}
	
	if ((int)colorPref1*(int)colorPref2>0) /* table perturbee */
	{
		if ((Perturbation.hight==0) && (Perturbation.low==0))
		{
			STD_LOG("testMatch")("pas de perturbation");
			return (1); /* pas de perturbation */
		}
		if ( (abs(colorPref1)==3) && (abs(colorPref2)==3) )
		{
			STD_LOG("testMatch")("2 pref absolues identiques");
			return (1); /* 2 preferences absolues */
		}
		if ( (abs(colorPref1)>=2) && (abs(colorPref2)>=2) && (Perturbation.hight==0))
		{
			STD_LOG("testMatch")("Pref haute a ne pas perturbees");
			return (1); /* ne change pas les preferences hautes */
		}
		if (( ((Perturbation.color==BLACK) && (colorPref1<0)) ||
			 ((Perturbation.color==WHITE) && (colorPref1>0)) )
			&& (0==param.ruleC11) )
		{
			STD_LOG("testMatch")("pas la bonne couleur a perturbee");
			return (1); /* pas la bonne couleur a perturbee */
		}
	}
	/* verification B5 */
	if (param.ruleB5)
	{
		DET_LOG("testMatch")("verification regle B5");
		if ((g_round<g_nbRound) || (player1.score<g_nbRound/2))
		{
			if ((player1.flotteur[g_round-1]==1) &&
			    (player1.score<player2.score))
			{
				STD_LOG("testMatch")("pb flotteur B5");
				return (1);
			}
		}
		if ((g_round<g_nbRound) || (player2.score<g_nbRound/2))
		{
			if ((player2.flotteur[g_round-1]==1) &&
			    (player2.score<player1.score))
			{
				STD_LOG("testMatch")("pb flotteur B5");
				return (1);
			}
		}
		/* verification B6 */
		if (param.ruleB6)
		{
			DET_LOG("testMatch")(" et regle B6");
			if ((g_round<g_nbRound) || (player1.score<g_nbRound/2))
			{
				if ((player1.flotteur[g_round-2]==1) 
				     && (player1.score<player2.score))
				return (1);
				if ((player1.flotteur[g_round-2]==-1) 
				     && (player1.score>player2.score))
				return (1);
			}
			if ((g_round<g_nbRound) || (player2.score<g_nbRound/2))
			{
				if ((player2.flotteur[g_round-2]==1) 
				     && (player2.score<player1.score))
				return (1);
				if ((player2.flotteur[g_round-2]==-1) 
				     && (player2.score>player1.score))
				return (1);
			}
		}
	}
	STD_LOG("testMatch")("return OK");
	return (OK);
}

/**
 * détermine la couleur des joueurs dans le match player0-player1
 *\param struct player *player0
 *\param struct player *player1
 *\return 0
 */
int getColorMatch(struct player *player0, struct player *player1)
{
	if (colorPref(*player0)*colorPref(*player1)<0)
	{
		if (colorPref(*player0)<0)
			player0->color[g_round]=WHITE;
		else
			player0->color[g_round]=BLACK;
		player1->color[g_round]=-player0->color[g_round];
	}
	else
	{
		if (abs(colorPref(*player0))>abs(colorPref(*player1)))
		{
			if (colorPref(*player0)<0)
				player0->color[g_round]=WHITE;
			else
				player0->color[g_round]=BLACK;
			player1->color[g_round]=-player0->color[g_round];
		}
		else
			if (abs(colorPref(*player0))<abs(colorPref(*player1)))
			{
				if (colorPref(*player1)<0)
					player0->color[g_round]=BLACK;
				else
					player0->color[g_round]=WHITE;
				player1->color[g_round]=-player0->color[g_round];
			}
			else
			{ /* recherche historique */
				int nb0=g_round-1, nb1=g_round-1;
				while (player0->color[nb0]==NOPLAY)
					nb0--;
				while (player1->color[nb1]==NOPLAY)
					nb1--;
				while ((player0->color[nb0]==player1->color[nb1]) 
					&& (nb0>1) && (nb1>1))
				{
					nb0--;
					nb1--;
					while (player0->color[nb0]==NOPLAY)
						nb0--;
					while (player1->color[nb1]==NOPLAY)
						nb1--;
				}
				if (player0->color[nb0]!=player1->color[nb1])
				{
					player0->color[g_round]=-player0->color[nb0];
					player1->color[g_round]=-player1->color[nb1];
				}
				else
				{
					if (bestPlayer(*player0,*player1)==1)
					{
						if (colorPref(*player0)<0)
						{
						    player0->color[g_round]=WHITE;
						    player1->color[g_round]=BLACK;
						}
						else
						{
						    player0->color[g_round]=BLACK;
						    player1->color[g_round]=WHITE;
						}
					}
					else
					{
						if (colorPref(*player0)<0)
						{
						    player0->color[g_round]=BLACK;
						    player1->color[g_round]=WHITE;
						}
						else
						{
						    player0->color[g_round]=WHITE;
						    player1->color[g_round]=BLACK;
						}
					}
				}
			}
	}
	player0->colorTheorique[g_round]=player0->color[g_round];
	player1->colorTheorique[g_round]=player1->color[g_round];
	return (OK);
}
